@@ -31,7 +31,7 @@ impl<'a, WI: Iterator<Item = &'a Arc<AlphanumRef>>, WFI: Iterator<Item = &'a Arc
 ///
 /// See [`strsim`] for more details on these algorithms,
 /// as that's the library which provide them.
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Algorithm {
     /// about 10x as slow as [`Self::Exact`],
     /// but actually accepts similar words.
@@ -234,7 +234,7 @@ pub struct ProximateDocItem<'a> {
 }
 impl<'a> PartialEq for ProximateDocItem<'a> {
     fn eq(&self, other: &Self) -> bool {
-        self.id == other.id
+        self.id == other.id && self.word == other.word
     }
 }
 impl<'a> Eq for ProximateDocItem<'a> {}
@@ -245,7 +245,12 @@ impl<'a> PartialOrd for ProximateDocItem<'a> {
 }
 impl<'a> Ord for ProximateDocItem<'a> {
     fn cmp(&self, other: &Self) -> std::cmp::Ordering {
-        self.id.cmp(&other.id)
+        let cmp = self.id.cmp(&other.id);
+        if cmp.is_eq() {
+            self.word.cmp(other.word)
+        } else {
+            cmp
+        }
     }
 }
 impl<'a> ProximateDocItem<'a> {
@@ -274,21 +279,22 @@ pub struct ProximateDocIter<'a, P: Provider<'a>> {
 impl<'a, P: Provider<'a>> Iterator for ProximateDocIter<'a, P> {
     type Item = ProximateDocItem<'a>;
     fn next(&mut self) -> Option<Self::Item> {
-        if let Some((word, doc_iter, proximity)) = &mut self.current {
-            if let Some(doc) = doc_iter.next() {
-                Some(ProximateDocItem::new((doc, word, *proximity)))
-            } else {
+        loop {
+            if let Some((word, doc_iter, proximity)) = &mut self.current {
+                if let Some(doc) = doc_iter.next() {
+                    return Some(ProximateDocItem::new((doc, word, *proximity)));
+                }
                 self.current = None;
-                self.next()
+                continue;
             }
-        } else if let Some((next_word, proximity)) = self.word_iter.next() {
-            self.current = self
-                .provider
-                .documents_with_word(&**next_word)
-                .map(|iter| (next_word, iter, *proximity));
-            self.next()
-        } else {
-            None
+            if let Some((next_word, proximity)) = self.word_iter.next() {
+                self.current = self
+                    .provider
+                    .documents_with_word(&**next_word)
+                    .map(|iter| (next_word, iter, *proximity));
+                continue;
+            }
+            return None;
         }
     }
 }
