@@ -11,8 +11,8 @@ use std::iter::Peekable;
 use crate::index::{self, AssociatedOccurrence};
 use crate::index::{Occurence, Provider};
 use crate::proximity::ProximateMap;
-use crate::set::ProgressiveInclusion;
 use crate::{proximity, set};
+use iter_set::Inclusion;
 pub use parse::{parse, Options as ParseOptions};
 
 #[derive(Debug, Clone)]
@@ -239,8 +239,11 @@ impl<'a, 'b, P: Provider<'a>> Documents<'a, 'b, P> {
             &set::union,
         )
     }
+    /// The list of proximate words.
+    /// This is populated on creation and used in [`Self::iter`].
+    /// Using [`Self::iter`] after calling this will most likely result in a panic.
     pub fn take_proximate_map(&mut self) -> ProximateMap<'b> {
-        std::mem::take(&mut self.proximate_map)
+        core::mem::take(&mut self.proximate_map)
     }
 }
 
@@ -462,14 +465,23 @@ impl Query {
                             Some(abs_diff_occurrence),
                         )
                         .filter_map(|inclusion| match inclusion {
-                            ProgressiveInclusion::Left(and) => Some(and),
-                            ProgressiveInclusion::Right(_not) => None,
-                            ProgressiveInclusion::Both(mut and, not) => {
+                            Inclusion::Left(mut and) => {
+                                *and.0.rating_mut() += 2.5;
+                                Some(and)
+                            }
+                            Inclusion::Right(_not) => None,
+                            Inclusion::Both(mut and, not) => {
+                                let not_rating = not.0.rating();
                                 let closest = and.closest(&OccurenceEq::new(not.0.occurrence, 0));
                                 // Don't really care about precision.
                                 #[allow(clippy::cast_precision_loss)]
                                 let rating_decrease = 1.0 / (0.0001 * closest.0 as f32 + 0.025);
                                 *and.0.rating_mut() -= rating_decrease;
+                                // by decreasing the rating based on the not rating too, we remove
+                                // extra when there are multiple NOTs.
+                                // `TODO` apply the same filter as we do below for acceted ones for
+                                // the NOT occurrences, so groups are better recognized.
+                                *and.0.rating_mut() -= not_rating;
                                 and.0.closest_not = Some(closest.1);
                                 Some(and)
                             }
@@ -489,7 +501,7 @@ impl Query {
                         None,
                     )
                     .filter_map(|inclusion| match inclusion {
-                        ProgressiveInclusion::Both(mut a, b) => {
+                        Inclusion::Both(mut a, b) => {
                             a.0.merge(&b.0);
                             Some(a)
                         }
@@ -509,47 +521,52 @@ impl Query {
                         None,
                     )
                     .map(|inclusion| match inclusion {
-                        ProgressiveInclusion::Both(mut a, b) => {
+                        Inclusion::Both(mut a, b) => {
                             a.0.merge(&b.0);
                             a
                         }
-                        ProgressiveInclusion::Left(a) | ProgressiveInclusion::Right(a) => a,
+                        Inclusion::Left(a) | Inclusion::Right(a) => a,
                     })
                 },
             )
             .map(|iter| {
                 iter.map(|occ| {
                     let mut occ = occ.0;
+
+                    let mut increase = 0.;
+                    // We keep track of the closest to determine which index should be the `major`
+                    // one.
                     let (mut closest, mut closest_index) = (usize::MAX, 0);
+
                     let mut iter = occ.occurrences();
                     let mut last = iter
                         .next()
                         .unwrap_or_else(|| AssociatedOccurrence::new(0, 0));
-                    for (idx, occ) in iter.enumerate() {
-                        if last.word_id() != occ.word_id() {
-                            let dist = occ.start() - last.start();
+
+                    for (idx, associated_occ) in iter.enumerate() {
+                        if last.word_id() != associated_occ.word_id() {
+                            let dist = associated_occ.start() - last.start();
                             if dist < closest {
-                                closest_index = idx + 1;
+                                closest_index = idx;
                             }
                             closest = std::cmp::min(dist, closest);
+                            // Don't really care about precision.
+                            #[allow(clippy::cast_precision_loss)]
+                            let rating_increase = 0.5 / (0.001 * dist as f32 + 0.1);
+                            increase += rating_increase;
                         }
-                        last = occ;
+                        last = associated_occ;
                     }
-                    if closest < usize::MAX {
-                        // Don't really care about precision.
-                        #[allow(clippy::cast_precision_loss)]
-                        let rating_increase = 1.0 / (0.001 * closest as f32 + 0.1);
-                        *occ.rating_mut() += rating_increase;
+                    *occ.rating_mut() += increase;
 
-                        // replace `occurrence.start()` with the closest match.
-                        // This requires removing the closest from the set and setting as "main"
-                        // and inserting "main" to the set.
-                        if closest_index != 0 {
-                            let closest = occ.occurrences().nth(closest_index - 1).unwrap();
-                            occ.occurrences.remove(&closest);
-                            occ.occurrences.insert((&occ.occurrence).into());
-                            *occ.occurrence.start_mut() = closest.start();
-                        }
+                    // replace `occurrence.start()` with the closest match.
+                    // This requires removing the closest from the set and setting as "main"
+                    // and inserting "main" to the set.
+                    if closest_index != 0 {
+                        let closest = occ.occurrences().nth(closest_index).unwrap();
+                        occ.occurrences.remove(&closest);
+                        occ.occurrences.insert((&occ.occurrence).into());
+                        *occ.occurrence.start_mut() = closest.start();
                     }
 
                     occ
@@ -691,6 +708,10 @@ pub mod parse {
         }
     }
 
+    /// Parse `s` with `opts`.
+    ///
+    /// You can also use the [`FromStr`] trait implemented by [`Query`] and [`Part`]
+    /// if you don't care about the [`Options`].
     #[allow(clippy::missing_panics_doc)]
     #[allow(clippy::missing_errors_doc)]
     pub fn parse(s: &str, opts: Options) -> Result<Part, Error> {
@@ -976,6 +997,12 @@ pub mod parse {
         rules: Vec<Box<dyn Rule>>,
     }
     impl Options {
+        /// Crates a new, empty, set of options.
+        /// The order of [`Self::insert`] matters a lot.
+        ///
+        /// [`Self::populate_and_space`] should definitely be last.
+        ///
+        /// See [`literal_rule!`] for an example.
         pub fn new() -> Self {
             Self { rules: Vec::new() }
         }
@@ -983,16 +1010,24 @@ pub mod parse {
             self.rules.push(Box::new(rule));
             self
         }
+        pub fn populate_literals(self) -> Self {
+            self.insert(NotLiteral::default())
+                .insert(AndLiteral::default())
+                .insert(OrLiteral::default())
+        }
+        pub fn populate_not(self) -> Self {
+            self.insert(DashNot::default()).insert(BangNot::default())
+        }
+        pub fn populate_and_space(self) -> Self {
+            self.insert(AndSpace::default())
+        }
     }
     impl Default for Options {
         fn default() -> Self {
             Self::new()
-                .insert(NotLiteral::default())
-                .insert(AndLiteral::default())
-                .insert(OrLiteral::default())
-                .insert(DashNot::default())
-                .insert(BangNot::default())
-                .insert(AndSpace::default())
+                .populate_literals()
+                .populate_not()
+                .populate_and_space()
         }
     }
     pub trait Rule: Debug {
@@ -1039,8 +1074,24 @@ pub mod parse {
         }
     }
 
+    /// Used in [parsing](self).
+    ///
+    /// # Examples
+    ///
+    /// See [`literal_rule!`] and [`not_prefix!`].
+    ///
+    /// ```
+    /// use elipdotter::*;
+    /// use elipdotter::query::Part;
+    /// use elipdotter::query::parse::LiteralRule;
+    /// generate_rule!(EllerLiteral, LiteralRule, LiteralRule::new("eller", query::parse::Op::Or));
+    /// let opts = elipdotter::query::ParseOptions::new().populate_literals().insert(EllerLiteral::default()).populate_not().populate_and_space();
+    /// let part = elipdotter::query::parse("elipdotter eller search", opts).unwrap();
+    /// assert_eq!(part, Part::or("elipdotter", "search"));
+    /// ```
+    #[macro_export]
     macro_rules! generate_rule {
-        ($name: ident, $struct: ident, $new: expr) => {
+        ($name: ident, $struct: path, $new: expr) => {
             #[derive(Debug)]
             #[must_use]
             pub struct $name($struct);
@@ -1054,8 +1105,12 @@ pub mod parse {
                     Self::new()
                 }
             }
-            impl Rule for $name {
-                fn next(&mut self, parser: &mut Parser, rest: &str) -> Option<usize> {
+            impl $crate::query::parse::Rule for $name {
+                fn next(
+                    &mut self,
+                    parser: &mut $crate::query::parse::Parser,
+                    rest: &str,
+                ) -> Option<usize> {
                     self.0.next(parser, rest)
                 }
             }
@@ -1067,23 +1122,30 @@ pub mod parse {
     pub struct LiteralRule {
         literal: &'static str,
         op: Op,
+        last_was_space: bool,
     }
     impl LiteralRule {
         /// Matches the `literal` with [`Op`].
         ///
         /// Will exit if nothing was detected before and this is a binary operation.
         pub fn new(literal: &'static str, op: Op) -> Self {
-            Self { literal, op }
+            Self {
+                literal,
+                op,
+                last_was_space: true,
+            }
         }
     }
     impl Rule for LiteralRule {
         fn next(&mut self, parser: &mut Parser, rest: &str) -> Option<usize> {
             if self.op.binary() && parser.string.is_empty() && parser.left.is_none() {
+                self.last_was_space = rest.chars().next().map_or(false, is_whitespace);
                 return None;
             }
-            if rest
-                .get(0..self.literal.len())
-                .map_or(false, |rest| rest.eq_ignore_ascii_case(self.literal))
+            let rule = if self.last_was_space
+                && rest
+                    .get(0..self.literal.len())
+                    .map_or(false, |rest| rest.eq_ignore_ascii_case(self.literal))
                 && rest
                     .chars()
                     .nth(self.literal.len())
@@ -1093,14 +1155,34 @@ pub mod parse {
                 Some(self.literal.len())
             } else {
                 None
-            }
+            };
+
+            self.last_was_space = rest.chars().next().map_or(false, is_whitespace);
+
+            rule
         }
     }
 
+    /// Used in [parsing](self).
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use elipdotter::*;
+    /// use elipdotter::query::Part;
+    /// literal_rule!(EllerLiteral, "eller", query::parse::Op::Or);
+    /// let opts = elipdotter::query::ParseOptions::new().populate_literals().insert(EllerLiteral::default()).populate_not().populate_and_space();
+    /// let part = elipdotter::query::parse("elipdotter eller search", opts).unwrap();
+    /// assert_eq!(part, Part::or("elipdotter", "search"));
+    /// ```
     #[macro_export]
     macro_rules! literal_rule {
         ($name: ident, $literal: expr, $op: expr) => {
-            generate_rule!($name, LiteralRule, LiteralRule::new($literal, $op));
+            $crate::generate_rule!(
+                $name,
+                $crate::query::parse::LiteralRule,
+                $crate::query::parse::LiteralRule::new($literal, $op)
+            );
         };
     }
 
@@ -1137,10 +1219,26 @@ pub mod parse {
         }
     }
 
+    /// Used in [parsing](self).
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use elipdotter::*;
+    /// use elipdotter::query::Part;
+    /// not_prefix!(HyphenNot, "-");
+    /// let opts = elipdotter::query::ParseOptions::default().insert(HyphenNot::default());
+    /// let part = elipdotter::query::parse("elipdotter -search", opts).unwrap();
+    /// assert_eq!(part, Part::and("elipdotter", Part::not("search")));
+    /// ```
     #[macro_export]
     macro_rules! not_prefix {
         ($name: ident, $prefix: expr) => {
-            generate_rule!($name, NotPrefix, NotPrefix::new($prefix));
+            $crate::generate_rule!(
+                $name,
+                $crate::query::parse::NotPrefix,
+                $crate::query::parse::NotPrefix::new($prefix)
+            );
         };
     }
     not_prefix!(DashNot, "-");
@@ -1194,6 +1292,13 @@ mod tests {
         assert_eq!(part, Part::not("icelk"));
     }
     #[test]
+    fn parse_plain_or() {
+        let p = s("or");
+        assert_eq!(p, Part::s("or"));
+        let p = s("for me");
+        assert_eq!(p, Part::and("for", "me"));
+    }
+    #[test]
     fn parse_empty() {
         assert_eq!(
             parse("", ParseOptions::default()).unwrap_err(),
@@ -1301,16 +1406,17 @@ mod tests {
                     Part::and("agde", Part::not("sync")),
                     Part::and("icelk", Part::not("kvarn"))
                 ),
-                Part::s("agde")
+                "agde"
             )
         );
     }
     #[test]
     fn parse_prefix_not() {
         assert_eq!(s("icelk !kvarn"), s("icelk -kvarn"));
+        assert_eq!(s("icelk !kvarn"), Part::and("icelk", Part::not("kvarn")));
         assert_eq!(
-            s("icelk !kvarn"),
-            Part::and(Part::s("icelk"), Part::not("kvarn"))
+            s("elipdotter -search"),
+            Part::and("elipdotter", Part::not("search"))
         );
     }
     #[test]
@@ -1318,10 +1424,7 @@ mod tests {
         assert_eq!(s("icelk.dev"), Part::s("icelkdev"));
         assert_eq!(
             s("next-generation kvarn"),
-            Part::and(
-                Part::and(Part::s("next"), Part::s("generation")),
-                Part::s("kvarn")
-            )
+            Part::and(Part::and("next", "generation"), "kvarn")
         );
     }
 
